@@ -109,6 +109,15 @@ struct DWCComputeKernelInfo
     bool         export_weights_to_cl_image{ false }; /**< Export the weights to cl_image */
 };
 
+/** Compute descriptor used by the direct convolution kernel */
+struct DirectConvComputeKernelInfo
+{
+    int32_t m0{ 1 };                             /**< Number of rows to be processed by the kernel */
+    int32_t n0{ 1 };                             /**< Number of columns to be processed by the kernel */
+    int32_t k0{ 1 };                             /**< Number of partial accumulations to be processed in a single iteration by the kernel */
+    bool    export_weights_to_cl_image{ false }; /**< Flag to export the weights to cl_image */
+};
+
 /** Descriptor used by the softmax kernels */
 struct SoftmaxKernelInfo
 {
